// (C) 2017-2020 by folkert van heusden, released under AGPL v3.0
#pragma once

#include <atomic>
#include <pthread.h>
#include <stdint.h>
#include <thread>

#include "interface.h"
#include "resize.h"
#include "error.h"

typedef enum { E_RGB, E_JPEG } encoding_t;

class filter;

class source : public interface
{
protected:
	int width, height;
	const double max_fps;
	resize *const r;
	const int resize_w, resize_h, loglevel;
	const double timeout;
	std::vector<filter *> *const filters;
	const std::string exec_failure;

	pthread_mutex_t lock;
	pthread_cond_t cond;
	uint64_t ts{ 0 };
	uint8_t *frame_jpeg{ nullptr }, *frame_rgb{ nullptr };
	size_t frame_jpeg_len{ 0 }, frame_rgb_len{ 0 };

	std::atomic_int user_count;

	int fps_counts[5]{ 0 }, last_fps_count_sec{ 0 };
	double cur_fps{ 0.0 };

	void do_exit_failure();

	source(const std::string & id, const std::string & descr, const std::string & exec_failure, const int w, const int h, resize *const r);
	source(const std::string & id, const std::string & descr, const std::string & exec_failure, const int w, const int h, resize *const r, std::vector<filter *> *const filters);
	source(const std::string & id, const std::string & descr, const std::string & exec_failure, const double max_fps, const int w, const int h, const int loglevel, std::vector<filter *> *const filters);
	void init();

	void track_fps(const struct timespec *const tv);

public:
	source(const std::string & id, const std::string & descr, const std::string & exec_failure, const double max_fps, resize *const r, const int resize_w, const int resize_h, const int loglevel, const double timeout, std::vector<filter *> *const filters);
	virtual ~source();

	virtual bool get_frame(const encoding_t pe, const int jpeg_quality, uint64_t *ts, int *width, int *height, uint8_t **frame, size_t *frame_len, const bool repeat);
	void set_frame(const encoding_t pe, const uint8_t *const data, const size_t size);
	void set_scaled_frame(const uint8_t *const in, const int sourcew, const int sourceh);
	void set_size(const int w, const int h) { width = w; height = h; }
	bool need_scale() const { return resize_h != -1 || resize_w != -1; }

	virtual uint64_t get_current_ts();
	double get_fps();

	int get_width() const { return width; }
	int get_height() const { return height; }

	virtual void operator()() { }

	virtual void register_user() { user_count++; }
	virtual void unregister_user() { user_count--; }
	bool work_required();
};
